

// Requisite headers
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include "gl_saver.h"

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// Do any initialization of the rendering context here, such as
// setting background colors, setting up lighting, or performing
// preliminary calculations.
void GLSetupRC(void *pData)
	{
	// Light values and coordinates
	GLfloat  ambientLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat  diffuseLight[] = { 0.6f, 0.6f, 0.6f, 1.0f };
	GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat	 lightPos[] = { -150.0f, 150.0f, 150.0f, 1.0f };
	GLfloat  specref[] =  { 1.0f, 1.0f, 1.0f, 1.0f };

	// Kill the back faces (this is why we don't need depth test for a sphere)
	glDisable(GL_DEPTH_TEST);	// Don't need depth testing
	glEnable(GL_CULL_FACE);
	glFrontFace(GL_CCW);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f );

	// Setup and enable light 0
	glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
	glEnable(GL_LIGHT0); 

	// Enable color tracking
	glEnable(GL_COLOR_MATERIAL);
	
	// Set Material properties to follow glColor values
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glColor3f(1.0f, 0.0f, 0.3f);

	// All materials hereafter have full specular reflectivity
	// with a high shine
	glMaterialfv(GL_FRONT, GL_SPECULAR,specref);
	glMateriali(GL_FRONT,GL_SHININESS,128);

 }


//////////////////////////////////////////////////////////////////
// Shutdown the rendering context
void GLShutdownRC(void)
	{

	}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// Render the OpenGL Scene here. Called by the WM_PAINT message
// handler.
void GLRenderScene(void *pData)
{
	static float i =0;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;

	// ***** Put drawing code here. *****
	// Here's some to get you started....

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	

	// Reset The Projection Matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();							

	gluPerspective(60.0f,
			(GLfloat)1024/(GLfloat)768,
			0.1f,
			1000.0f);

	// Move back a little so we can see it.
	glTranslatef(0,0,-20);

	i+= 0.4 ;
	glRotatef(i,1,1,1);

	
	auxWireTeapot(5);
	
	// Flush drawing commands
	glFlush();
}


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// Called by Windows when it receives the WM_SIZE message.
// Put any code needed here to recalc the viewing volume and
// viewport info. For a screen saver, this only occurs once.
void GLResize(GLsizei w, GLsizei h)
	{
	GLfloat nRange = 100.0f;
	GLfloat fAspect;

	// Prevent a divide by zero
	if(h == 0)
		h = 1;

	fAspect = (GLfloat)w/(GLfloat)h;

	// Set Viewport to window dimensions
	glViewport(0, 0, w, h);

	glMatrixMode(GL_PROJECTION);

	// Reset coordinate system
	glLoadIdentity();

	// Setup perspective for viewing
	gluPerspective(45.0f,fAspect,1,1070);

	// Viewing transformation
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();

}

