
// Required Headers
#include <windows.h>
#include <scrnsave.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include "gl_saver.h"

/////////////////////////////////////////////////////////////////////////////////
// GLOBAL Variables for this file
HGLRC hRC = (HGLRC) 0;
HDC hDC = (HDC) 0;
HPALETTE hPalette = (HPALETTE) 0;

// Timer ID's
UINT nTimerID = 101;
#define  DLG_SCRNSAVECONFIGURE 2003 

///////////////////////////////////////////////////////////////////////////////
// FUNCTION PROTOTYPES
void InitializeGLPalette(void);
LRESULT WINAPI ScreenSaverChildProc(HWND  hWnd,UINT  message, WPARAM  wParam, LPARAM  lParam);
LRESULT WINAPI ScreenSaverProc(HWND  hWnd,UINT  message, WPARAM  wParam, LPARAM  lParam);

///////////////////////////////////////////////////////////////////////////////
// Process Messages for the main Window
LRESULT WINAPI ScreenSaverProc(HWND  hWnd,UINT  message, WPARAM  wParam, LPARAM  lParam)
{
	static HWND hChild = NULL;
	HINSTANCE hInst = hMainInstance;
	int nMaxX = 0;
	int nMaxY = 0;
	int nWndSize = 0;
    switch (message)
	{
	case WM_CREATE:
		{
			// Create the child window
			WNDCLASS        wc;                     // Windows class structure
			
			// Register Window style
			wc.style                = CS_VREDRAW | CS_HREDRAW | CS_OWNDC ;
			wc.lpfnWndProc          = (WNDPROC) ScreenSaverChildProc;
			wc.cbClsExtra           = 0;
			wc.cbWndExtra           = 0;
			wc.hInstance            = hInst;
			wc.hIcon                = NULL;
			wc.hCursor              = NULL;
			
			// No need for background brush for OpenGL window
			wc.hbrBackground        = NULL;
			
			wc.lpszMenuName         = NULL;
			wc.lpszClassName        = "NTDEVCLASS";
			
			// Register the window class
			RegisterClass(&wc);
			
			// Get the desktop size
			if(nMaxX == 0)
			{
				RECT rect;
				GetClientRect(hWnd,&rect);
				HDC hdc;
				hdc = GetDC(hWnd);
				nMaxX = rect.right;
				nMaxY = rect.bottom;
				nWndSize = (int) ((float)nMaxY* .35f);
				::ReleaseDC(hWnd,hdc);
			}
			
			
			// Create the main application window
			hChild = CreateWindow(
				wc.lpszClassName,
				"Child1",
				
				// OpenGL requires WS_CLIPCHILDREN and WS_CLIPSIBLINGS
				WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
				
				// Window position and size
				0, 0,
				nMaxX,nMaxY, //nWndSize, nWndSize,
				hWnd,
				NULL,
				hInst,
				NULL);
			
			
		}
		break;
		
	case WM_DESTROY:
		KillTimer(hWnd, nTimerID);
		DestroyWindow(hChild);
		PostQuitMessage(0);
		break;
		
	case WM_TIMER:
		
		break;
		
		
	default:
		return DefScreenSaverProc(hWnd, message, wParam, lParam);
	}
    return 0;
}


///////////////////////////////////////////////////////////////////////////////
// Process Messages for the Child (OpenGL) Window
LRESULT WINAPI ScreenSaverChildProc(HWND  hWnd,UINT  message, WPARAM  wParam, LPARAM  lParam)
{
    HDC hdc;
    PAINTSTRUCT ps;
    static BOOL bInited = FALSE;
	int nPixelFormat;					// Pixel format index
	int retVal;
	
	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// Size of this structure
			1,								// Version of this structure	
			PFD_DRAW_TO_WINDOW |			// Draw to Window (not to bitmap)
			PFD_SUPPORT_OPENGL |			// Support OpenGL calls in window
			PFD_DOUBLEBUFFER |   			// Double buffered mode
			PFD_DEPTH_DONTCARE,				// Dont' need depth buffer
			PFD_TYPE_RGBA,					// RGBA Color mode
			24,								// Want 24bit color if available
			0,0,0,0,0,0,					// Not used to select mode
			0,0,							// Not used to select mode
			0,0,0,0,0,						// Not used to select mode
			0,								// Size of depth buffer
			0,								// Not used to select mode
			0,								// Not used to select mode
			PFD_MAIN_PLANE,					// Draw in main plane
			0,								// Not used to select mode
			0,0,0 };						// Not used to select mode
		
		
		switch (message)
		{
        case WM_CREATE:
			hDC = GetDC(hWnd);
			
			// Choose a pixel format that best matches that described in pfd
			nPixelFormat = ChoosePixelFormat(hDC, &pfd);
			
			// Set the pixel format for the device context
			SetPixelFormat(hDC, nPixelFormat, &pfd);
			
			// Test pixel format to see if HW/Accelerated
			DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
			
				
				// Create the palette if needed
				InitializeGLPalette();
				
				// Create the rendering context
				hRC = wglCreateContext(hDC);
				
				// Make the rendering context current, perform initialization, then
				// deselect it
				wglMakeCurrent(hDC,hRC);
				
				GLSetupRC(hDC);
				
				// Set Timer to move child window
				SetTimer(hWnd, nTimerID,33,NULL); // About 30 fps max
				
				break;
				
        case WM_DESTROY:
			KillTimer(hWnd, nTimerID);
			
			// Clean up rendering context
			GLShutdownRC();
			wglMakeCurrent(hDC, NULL);
			wglDeleteContext(hRC);
			ReleaseDC(hWnd,hDC);
			
            PostQuitMessage(0);
            break;
			
		case WM_TIMER:
			InvalidateRect(hWnd, NULL, FALSE);
			break;
			
		case WM_ERASEBKGND:
			return 1;
			break;
			
		case WM_SIZE:
			GLResize(LOWORD(lParam), HIWORD(lParam));
			break;
			
			
        case WM_PAINT:
            hdc = BeginPaint(hWnd, &ps);
			
			// Call our external OpenGL code
			GLRenderScene(NULL);
			
			// Swap our scene to the front
			SwapBuffers(hDC);
			
            EndPaint(hWnd, &ps);
            break;
			
        case WM_SYSCOMMAND:
        case WM_SETCURSOR:
        case WM_NCACTIVATE:
        case WM_ACTIVATE:
        case WM_ACTIVATEAPP:
        case WM_MOUSEMOVE:
        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
        case WM_RBUTTONDOWN:
        case WM_KEYDOWN:
        case WM_SYSKEYDOWN:
            retVal = DefScreenSaverProc(hWnd, message, wParam, lParam);
            DefScreenSaverProc(hMainWindow, message, wParam, lParam);
            return retVal;
			
        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}
	
	

///////////////////////////////////////////////////////////////////////////////
// Configuration dialog
BOOL WINAPI ScreenSaverConfigureDialog (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
	case WM_INITDIALOG:
		return TRUE;
		break;
		
		
	case WM_COMMAND:
		switch (LOWORD(wParam)) 
		{
		case IDOK:
			EndDialog(hDlg, TRUE);
			break;
			
		default:
			break;
		}
		
		return TRUE;
		break;
		
		default:
			return 0;
			break;
	}
	
	return 0;
}


///////////////////////////////////////////////////////////////////////////////
// No Special Needs for special controls, etc.
BOOL WINAPI RegisterDialogClasses(HANDLE hInst)
{
	return TRUE;
}



///////////////////////////////////////////////////////////////////////////
// Initializes the 332 palette (only used on 256 color displays)
void InitializeGLPalette(void)
{
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
	LOGPALETTE *pPal;			// Pointer to memory for logical palette
	int nPixelFormat;			// Pixel format index
	int nColors;				// Number of entries in palette
	int i;						// Counting variable
	BYTE RedRange,GreenRange,BlueRange;
	// Range for each color entry (7,7,and 3)
	
	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(hDC);
	DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
	
	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return;
	
	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;	
	
	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));
	
	// Fill in palette header 
	pPal->palVersion = 0x300;		// Windows 3.0
	pPal->palNumEntries = nColors; // table size
	
	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;
	
	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
	{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);
		
		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);
		
		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);
		
		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
	}
	
	// Create the palette
	hPalette = CreatePalette(pPal);
	
	// Go ahead and select and realize the palette for this device context
	SelectPalette(hDC,hPalette,FALSE);
	RealizePalette(hDC);
	
	// Free the memory used for the logical palette structure
	free(pPal);
}
